'use strict';

// const debug = require('debug')
// const log = debug('libp2p:tcp')

var SWS = require('simple-websocket');
var mafmt = require('mafmt');

exports = module.exports = WebSockets;

function WebSockets() {
  if (!(this instanceof WebSockets)) {
    return new WebSockets();
  }

  var listeners = [];

  this.dial = function (multiaddr, options) {
    if (!options) {
      options = {};
    }

    options.ready = options.ready || function noop() {};
    var maOpts = multiaddr.toOptions();
    var conn = new SWS('ws://' + maOpts.host + ':' + maOpts.port);
    conn.on('connect', options.ready);
    conn.getObservedAddrs = function () {
      return [multiaddr];
    };
    return conn;
  };

  this.createListener = function (multiaddrs, options, handler, callback) {
    if (typeof options === 'function') {
      callback = handler;
      handler = options;
      options = {};
    }

    if (!Array.isArray(multiaddrs)) {
      multiaddrs = [multiaddrs];
    }

    var count = 0;

    multiaddrs.forEach(function (m) {
      var listener = SWS.createServer(function (conn) {
        conn.getObservedAddrs = function () {
          return []; // TODO think if it makes sense for WebSockets
        };
        handler(conn);
      });

      listener.listen(m.toOptions().port, function () {
        if (++count === multiaddrs.length) {
          callback();
        }
      });
      listeners.push(listener);
    });
  };

  this.close = function (callback) {
    if (listeners.length === 0) {
      callback(new Error('there are no listeners'));
    }
    var count = 0;
    listeners.forEach(function (listener) {
      listener.close(function () {
        if (++count === listeners.length) {
          callback();
        }
      });
    });
  };

  this.filter = function (multiaddrs) {
    return multiaddrs.filter(function (ma) {
      return mafmt.WebSockets.matches(ma);
    });
  };
}